import { QueryApi, WriteApi } from "@influxdata/influxdb-client";
import dotenv from "dotenv";
import { RouteManager, RouteMessages, createServer } from "server";
import { checkEnvironmentVariables, createInfluxClient } from "utils";
import { connectMongo } from "./db/mongo";
import NodeManager from "./node/nodeManager";
import ProxyRoute from "./routes/proxy";
import {
  INFLUXDB_BUCKET,
  INFLUXDB_ORG,
  INFLUXDB_TOKEN,
  INFLUXDB_URL,
  MONGO_URI,
  initSecrets,
} from "./secrets";

dotenv.config(); // load .env file

// Check environment variables
const envVarsValid = checkEnvironmentVariables("API_PORT", "INFISICAL_TOKEN");
if (!envVarsValid) {
  process.exit(1);
}

export let InfluxWriteApi: WriteApi;
export let InfluxQueryApi: QueryApi;

/**
 * The node manager for all of the loaded nodes
 */
export const nodeManager = new NodeManager();

(async () => {
  await initSecrets(process.env.INFISICAL_TOKEN!); // Load the infisical secrets

  // Init MongoDB
  await connectMongo(MONGO_URI);

  const routeManager = new RouteManager();
  routeManager.addRoute(new ProxyRoute());

  // Init InfluxDB
  const influxClient = createInfluxClient(
    INFLUXDB_TOKEN,
    INFLUXDB_URL,
    INFLUXDB_ORG,
    INFLUXDB_BUCKET
  );
  if (!influxClient) {
    console.log("Influx client failed to initialize, exiting...");
    process.exit(1);
  }
  InfluxWriteApi = influxClient.influxWriteClient;
  InfluxQueryApi = influxClient.influxQueryClient;

  const server = createServer({
    port: process.env.API_PORT || 3000,
  });

  server.all("*", (req, res) => {
    // Handle all paths to the proxy route
    const routes = routeManager.getRoutes();
    if (!routes || routes.length === 0 || !routes[0]) {
      res.status(500).json(RouteMessages.internalServerError("No routes"));
      return;
    }
    routes[0].handle(req, res);
  });
})();
